/* ************************************************************************** */
/* Includes definitions                                                       */
/* ************************************************************************** */
#include "detectarObjetos.h"

DetectarObjetos::DetectarObjetos()
{
	imgSize = cv::Size(320, 240);
	menuImg = cv::Mat(imgSize.height + _MENUBOTTOMHEIGHT, imgSize.width + _MENULEFTWIDTH, CV_8UC3, cv::Scalar(0, 0, 0));
	menutreegui.setMenuDim(imgSize.width + _MENULEFTWIDTH, imgSize.height + _MENUBOTTOMHEIGHT, _MENULEFTWIDTH, _MENUBOTTOMHEIGHT);
	showImg = cv::Mat(imgSize.height, imgSize.width, CV_8UC3, cv::Scalar(155));
	imgParaComparar = cv::Mat(imgSize.height, imgSize.width, CV_8UC1, cv::Scalar(155));
	showImg.setTo( cv::Scalar(155,0,0) );
	selectColorArea = cv::Rect(150, 110, 20, 20);
	key = 0;
	salir = false;
	operacion.clear();
	initMenu();
}

int DetectarObjetos::Exit()
{
	return (salir);
}

void DetectarObjetos::initMenu()
{
	menutreegui.addMenu("Informacion", "Use los cursores/intro para navegar");
	menutreegui.addMenu("Det. color", "Deteccion mediante sustraccion de fondo");
		menutreegui.addSubMenu("Mostrar area sel.", "Muestra el area para seleccionar color");
		menutreegui.addSubMenu("Seleccionar color", "Situe el objeto en el area de deteccion");
		menutreegui.addSubMenu("Deteccion color", "Detecta el color seleccionado");
		menutreegui.addSubMenu("Volver", "Vuelve al menu anterior");
	menutreegui.addMenu("Det. movimiento", "Detecteccion mediante sustraccion de fondo");
		menutreegui.addSubMenu("Capturar fondo", "Captura el fondo actual");
		menutreegui.addSubMenu("Det. fondo fijo", "Sustraccion con fondo fijo");
		menutreegui.addSubMenu("Det. fondo variable", "Sustraccion sin fondo fijo");
		menutreegui.addSubMenu("Volver", "Vuelve al menu anterior");
	menutreegui.addMenu("Salir", "Salir de la aplicacion");
		menutreegui.addSubMenu("Salir", "Salir de la aplicacion");
		menutreegui.addSubMenu("Volver", "Vuelve al menu anterior");
}

int DetectarObjetos::abrirCamara(int ncam)
{
	cv::namedWindow("Deteccion de objetos", CV_WINDOW_AUTOSIZE);
	if ( !streamCapture.open(ncam))
		return (0);
	streamCapture >> frame;
	if (frame.empty() )
		return (0);
	else
		cv::resize( frame, showImg , showImg.size(), 0, 0, cv::INTER_LINEAR );
	cv::Mat tmpImg = cv::Mat(menuImg, cv::Rect( _MENULEFTWIDTH, 0, imgSize.width, imgSize.height) );
	showImg.copyTo(tmpImg);
	menutreegui.drawLeftMenu(menuImg);
	menutreegui.drawBottomMenu(menuImg);	
	cv::imshow("Deteccion de objetos", menuImg);
	key = cv::waitKey(50);
	return (1);
}

int DetectarObjetos::update(int wait)
{
	streamCapture >> frame;
	if ( !frame.empty() )
		cv::resize( frame, showImg , showImg.size(), 0, 0, cv::INTER_LINEAR );
	else
		showImg.setTo( cv::Scalar(155,0,0) );
	leeEntradaDeUsuario();
	menutreegui.drawLeftMenu(menuImg);
	menutreegui.drawBottomMenu(menuImg);
	ejecutarOperacion();
	cv::Mat tmpImg = cv::Mat(menuImg, cv::Rect( _MENULEFTWIDTH, 0, imgSize.width, imgSize.height) );
	showImg.copyTo(tmpImg);
	cv::imshow("Deteccion de objetos", menuImg);
	key = cv::waitKey(wait);
	return (key);
}

void DetectarObjetos::ejecutarOperacion()
{
	menuDetectarMovimiento();
	menuDetectarColor();
}

void DetectarObjetos::menuDetectarColor()
{
	if ( operacion.compare("Mostrar area sel.") == 0 )
	{
		cv::rectangle(showImg, cv::Point(selectColorArea.x, selectColorArea.y), cv::Point(selectColorArea.x + selectColorArea.width, selectColorArea.y + selectColorArea.height), cv::Scalar(0,0,255), 1, CV_AA, 0);
	}
	if ( operacion.compare("Seleccionar color") == 0 )
	{
		cv::Mat tmpImg = cv::Mat(imgSize.height, imgSize.width, CV_8UC3, cv::Scalar(155));
		cv::cvtColor( showImg, tmpImg, CV_BGR2HSV );
		tmpImg = cv::Mat(tmpImg, selectColorArea );
		cv::meanStdDev(tmpImg, colorObjeto, colorObjetoDev);
		operacion = "";
	}
	if ( operacion.compare("Deteccion objetos") == 0 )
	{
		cv::Mat tmpImg = cv::Mat(imgSize.height, imgSize.width, CV_8UC3, cv::Scalar(155));
		cv::cvtColor( showImg, tmpImg, CV_BGR2HSV );
		cv::absdiff(tmpImg, colorObjeto, tmpImg);
		cv::Mat tmpImgGray = cv::Mat(imgSize.height, imgSize.width, CV_8UC1, cv::Scalar(155));
		std::vector<cv::Mat> splittedHSV;
		cv::split( tmpImg, splittedHSV );
		cv::threshold(splittedHSV[0], splittedHSV[0], colorObjetoDev[0]*2, 255, cv::THRESH_BINARY_INV);
		cv::threshold(splittedHSV[1], splittedHSV[1], colorObjetoDev[1]*2, 255, cv::THRESH_BINARY_INV);
		cv::threshold(splittedHSV[2], splittedHSV[2], colorObjetoDev[2]*3, 255, cv::THRESH_BINARY_INV);
		std::vector<cv::Mat> splitted;
		cv::split( showImg, splitted );
		splitted[1] += splittedHSV[0].mul( splittedHSV[1].mul(splittedHSV[2]) );
		cv::merge( splitted, showImg);
	}
}

void DetectarObjetos::menuDetectarMovimiento()
{
	if ( operacion.compare("Capturar fondo") == 0 )
		cv::cvtColor(showImg, imgParaComparar, CV_BGR2GRAY);
	if ( operacion.compare("Det. fondo fijo") == 0 )
	{
		cv::Mat tmpImg = cv::Mat(imgSize.height, imgSize.width, CV_8UC1, cv::Scalar(155));
		cv::cvtColor(showImg, tmpImg, CV_BGR2GRAY);
		cv::absdiff(tmpImg, imgParaComparar, tmpImg);
		cv::threshold(tmpImg, tmpImg, 5, 255, cv::THRESH_BINARY);
		std::vector<cv::Mat> splitted;
		cv::split( showImg, splitted );
		splitted[1] += tmpImg;
		cv::merge( splitted, showImg);
	}
	if ( operacion.compare("Det. fondo variable") == 0 )
	{
		cv::Mat tmpImg = cv::Mat(imgSize.height, imgSize.width, CV_8UC1, cv::Scalar(155));
		cv::cvtColor(showImg, tmpImg, CV_BGR2GRAY);
		cv::absdiff(tmpImg, imgParaComparar, tmpImg);
		tmpImg.convertTo(tmpImg, CV_8UC1);
		cv::threshold(tmpImg, tmpImg, 5, 255, cv::THRESH_BINARY);
		std::vector<cv::Mat> splitted;
		cv::split( showImg, splitted );
		splitted[1] += tmpImg;
		cv::cvtColor(showImg, imgParaComparar, CV_BGR2GRAY);
		cv::merge( splitted, showImg);
	}
}

void DetectarObjetos::leeEntradaDeUsuario()
{
	if ( key == 82)
		menutreegui.previousMenu();
	else if ( key == 84)
		menutreegui.nextMenu();
	else if ( key == 27 )
	{
		menutreegui.exitSubmenu();
	}
	else if ( key == 10)
	{
		std::string mSelected = menutreegui.getHighlightedMenu();
		if ( menutreegui.highlightedIndexHasSubmenu() )
			menutreegui.enterSubmenu();
		else if ( mSelected.compare("Volver") == 0 )
		{
			menutreegui.exitSubmenu();
			operacion.clear();
		}
		else if ( mSelected.compare("Salir") == 0 )
			salir = true;
		else
			operacion = mSelected;
	}
}
